@@ -1,3 +1,24 @@
+/*
+ * Copyright (C) 2020-2021 Matthieu Gautier <mgautier@kymeria.fr>
+ * Copyright (C) 2020 Emmanuel Engelhart <kelson@kiwix.org>
+ * Copyright (C) 2020 Veloman Yunkan
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 2 of the
+ * License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * is provided AS IS, WITHOUT ANY WARRANTY; without even the implied
+ * warranty of MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, and
+ * NON-INFRINGEMENT.  See the GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
+ *
+ */
+
 #ifndef _LIBZIM_COMPRESSION_
 #define _LIBZIM_COMPRESSION_
 
@@ -13,6 +34,7 @@
 #include <zstd.h>
 
 #include "zim_types.h"
+#include "constants.h"
 
 //#define DEB(X) std::cerr << __func__ << " " << X << std::endl ;
 #define DEB(X)
@@ -26,7 +48,6 @@ enum class CompStatus {
   OK,
   STREAM_END,
   BUF_ERROR,
-  OTHER
 };
 
 enum class RunnerStatus {
@@ -83,7 +104,7 @@ template<typename INFO>
 class Uncompressor
 {
   public:
-    Uncompressor(size_t initial_size=1024*1024) :
+    Uncompressor(size_t initial_size) :
       ret_data(new char[initial_size]),
       data_size(initial_size)
     {}
@@ -98,43 +119,49 @@ class Uncompressor
     RunnerStatus feed(char* data, size_t size, CompStep step = CompStep::STEP) {
       stream.next_in = (unsigned char*)data;
       stream.avail_in = size;
-      auto errcode = CompStatus::OTHER;
       while (true) {
-        errcode = INFO::stream_run_decode(&stream, step);
+        auto errcode = INFO::stream_run_decode(&stream, step);
         DEB((int)errcode)
-        if (errcode == CompStatus::BUF_ERROR) {
-          if (stream.avail_in == 0 && stream.avail_out != 0)  {
-            // End of input stream.
-            // compressor hasn't recognize the end of the input stream but there is
-            // no more input.
-            return RunnerStatus::NEED_MORE;
-          } else {
-            //Not enought output size
-            DEB("need memory " << data_size << " " << stream.avail_out << " " << stream.total_out)
-            data_size *= 2;
-            std::unique_ptr<char[]> new_ret_data(new char[data_size]);
-            memcpy(new_ret_data.get(), ret_data.get(), stream.total_out);
-            stream.next_out = (unsigned char*)(new_ret_data.get() + stream.total_out);
-            stream.avail_out = data_size - stream.total_out;
-            DEB(data_size << " " << stream.avail_out << " " << stream.avail_in)
-            ret_data = std::move(new_ret_data);
-            continue;
-          }
-        }
-        if (errcode == CompStatus::STREAM_END)
-          break;
-        // On first call where lzma cannot progress (no output size).
-        // Lzma return OK. If we return NEED_MORE, then we will try to compress
-        // with new input data, but we should not as current one is not processed.
-        // We must do a second step to have te BUF_ERROR and handle thing correctly.
-        if (errcode == CompStatus::OK) {
-          if (stream.avail_in == 0)
+        switch (errcode) {
+          case CompStatus::BUF_ERROR:
+            if (stream.avail_in == 0 && stream.avail_out != 0)  {
+              // End of input stream.
+              // compressor hasn't recognize the end of the input stream but there is
+              // no more input.
+              return RunnerStatus::NEED_MORE;
+            } else {
+              // Not enought output size.
+              // Allocate more memory and continue the loop.
+              DEB("need memory " << data_size << " " << stream.avail_out << " " << stream.total_out)
+              data_size *= 2;
+              std::unique_ptr<char[]> new_ret_data(new char[data_size]);
+              memcpy(new_ret_data.get(), ret_data.get(), stream.total_out);
+              stream.next_out = (unsigned char*)(new_ret_data.get() + stream.total_out);
+              stream.avail_out = data_size - stream.total_out;
+              DEB(data_size << " " << stream.avail_out << " " << stream.avail_in)
+              ret_data = std::move(new_ret_data);
+            }
             break;
-          continue;
+          case CompStatus::OK:
+            // On first call where lzma cannot progress (no output size).
+            // Lzma return OK. If we return NEED_MORE, then we will try to compress
+            // with new input data, but we should not as current one is not processed.
+            // We must do a second step to have te BUF_ERROR and handle thing correctly.
+            // If we have no more input, then we must ask for more.
+            if (stream.avail_in == 0) {
+              return RunnerStatus::NEED_MORE;
+            }
+            break;
+          case CompStatus::STREAM_END:
+            // End of compressed stream. Everything is ok.
+            return RunnerStatus::OK;
+          default:
+            // unreachable
+            return RunnerStatus::ERROR;
         }
-        return RunnerStatus::ERROR;
       };
-      return errcode==CompStatus::STREAM_END?RunnerStatus::OK:RunnerStatus::NEED_MORE;
+      // unreachable
+      return RunnerStatus::NEED_MORE;
     }
 
     std::unique_ptr<char[]> get_data(zim::zsize_t* size) {
@@ -156,7 +183,7 @@ class Uncompressor
  *
  * @param reader         The reader where the data is.
  * @param startOffset    The offset where the data is in the reader.
- * @param dest_size[out] The size of the uncompressed data.
+ * @param[out] dest_size The size of the uncompressed data.
  * @return A pointer to the uncompressed data. This must be deleted (delete[])
 */
 template<typename INFO>
@@ -164,8 +191,8 @@ std::unique_ptr<char[]> uncompress(const zim::Reader* reader, zim::offset_t star
   // Use a compressor to compress the data.
   // As we don't know the result size, neither the compressed size,
   // we have to do chunk by chunk until decompressor is happy.
-  // Let's assume it will be something like the minChunkSize used at creation
-  Uncompressor<INFO> runner(1024*1024);
+  // Let's assume it will be something like the default clusterSize used at creation
+  Uncompressor<INFO> runner(DEFAULT_CLUSTER_SIZE);
   // The input is a buffer of CHUNCK_SIZE char max. It may be less if the last chunk
   // is at the end of the reader and the reader size is not a multiple of CHUNCK_SIZE.
   std::vector<char> raw_data(CHUNCK_SIZE);
@@ -215,32 +242,39 @@ class Compressor
     RunnerStatus feed(const char* data, size_t size, CompStep step=CompStep::STEP) {
       stream.next_in = (unsigned char*)data;
       stream.avail_in = size;
-      auto errcode = CompStatus::OTHER;
       while (true) {
-        errcode = INFO::stream_run_encode(&stream, step);
-        if (stream.avail_out == 0) {
-          if (errcode == CompStatus::OK) {
-            // lzma return a OK return status the first time it runs out of output memory.
-            // The BUF_ERROR is returned only the second time we call a lzma_code.
-            continue;
-          }
-          if (errcode == CompStatus::BUF_ERROR) {
-            //Not enought output size
-            ret_size *= 2;
-            std::unique_ptr<char[]> new_ret_data(new char[ret_size]);
-            memcpy(new_ret_data.get(), ret_data.get(), stream.total_out);
-            stream.next_out = (unsigned char*)(new_ret_data.get() + stream.total_out);
-            stream.avail_out = ret_size - stream.total_out;
-            ret_data = std::move(new_ret_data);
-            continue;
-          }
-        }
-        if (errcode == CompStatus::STREAM_END || errcode == CompStatus::OK) {
-          // Everything ok, quit the loop
+        auto errcode = INFO::stream_run_encode(&stream, step);
+        switch (errcode) {
+          case CompStatus::OK:
+            if (stream.avail_out == 0) {
+              // lzma return a OK return status the first time it runs out of output memory.
+              // The BUF_ERROR is returned only the second time we call a lzma_code.
+              continue;
+            } else {
+              return RunnerStatus::NEED_MORE;
+            }
+          case CompStatus::STREAM_END:
+            return RunnerStatus::NEED_MORE;
+          case CompStatus::BUF_ERROR:
+            if (stream.avail_out == 0) {
+              //Not enought output size
+              ret_size *= 2;
+              std::unique_ptr<char[]> new_ret_data(new char[ret_size]);
+              memcpy(new_ret_data.get(), ret_data.get(), stream.total_out);
+              stream.next_out = (unsigned char*)(new_ret_data.get() + stream.total_out);
+              stream.avail_out = ret_size - stream.total_out;
+              ret_data = std::move(new_ret_data);
+              continue;
+            } else {
+              return RunnerStatus::ERROR;
+            }
           break;
-        }
-        return RunnerStatus::ERROR;
+          default:
+            // unreachable
+            return RunnerStatus::ERROR;
+        };
       };
+      // urreachable
       return RunnerStatus::NEED_MORE;
     }
 
