@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2017 Matthieu Gautier <mgautier@kymeria.fr>
+ * Copyright (C) 2017-2020 Matthieu Gautier <mgautier@kymeria.fr>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -22,13 +22,14 @@
 
 #include <iostream>
 #include <sstream>
+#include <stdexcept>
 #include <stdlib.h>
 
 #if defined (NDEBUG)
 # define ASSERT(left, operator, right) (void(0))
 #else
 
-#if !defined(_WIN32) && !defined(__APPLE__) && !defined(__ANDROID__)
+#if !defined(_WIN32) && !defined(__APPLE__) && !defined(__ANDROID__) && !defined(__EMSCRIPTEN__)
 #include <execinfo.h>
 #endif
 
@@ -40,7 +41,7 @@ void _on_assert_fail(const char* vara, const char* op, const char* varb,
       vara << "[" << a << "] " << op << " " << varb << "[" << b << "]";
   std::cerr << ss.str() << std::endl;
 
-#if !defined(_WIN32) && !defined(__APPLE__) && !defined(__ANDROID__)
+#if !defined(_WIN32) && !defined(__APPLE__) && !defined(__ANDROID__) && !defined(__EMSCRIPTEN__) && defined(__GNU_LIBRARY__)
   void *callstack[64];
   size_t size;
   size = backtrace(callstack, 64);
