@@ -1,4 +1,7 @@
 /*
+ * Copyright (C) 2019-2021 Matthieu Gautier <mgautier@kymeria.fr>
+ * Copyright (C) 2021 Maneesh P M <manu.pm55@gmail.com>
+ * Copyright (C) 2021 Veloman Yunkan
  * Copyright (C) 2009 Tommi Maekitalo
  *
  * This program is free software; you can redistribute it and/or
@@ -17,25 +20,31 @@
  *
  */
 
+#include <zim/writer/creator.h>
+
 #include "config.h"
 
 #include "creatordata.h"
 #include "cluster.h"
 #include "debug.h"
 #include "workers.h"
+#include "clusterWorker.h"
 #include <zim/blob.h>
-#include <zim/writer/creator.h>
+#include <zim/writer/contentProvider.h>
 #include "../endian_tools.h"
 #include <algorithm>
 #include <fstream>
 #include "../md5.h"
+#include "../constants.h"
+#include "counterHandler.h"
 
 #if defined(ENABLE_XAPIAN)
-  #include "xapianIndexer.h"
+# include "xapianHandler.h"
 #endif
 
 #ifdef _WIN32
 # include <io.h>
+# include <fcntl.h>
 #else
 # include <unistd.h>
 # define _write(fd, addr, size) if(::write((fd), (addr), (size)) != (ssize_t)(size)) \
@@ -62,134 +71,198 @@ log_define("zim.writer.creator")
     } while(false)
 
 #define TINFO(e) \
-    if (verbose) { \
+    if (m_verbose) { \
         double seconds = difftime(time(NULL), data->start_time); \
         std::cout << "T:" << (int)(seconds) \
                   << "; " << e << std::endl; \
     }
 
+#define TPROGRESS() \
+    if (m_verbose ) { \
+        double seconds = difftime(time(NULL),data->start_time);  \
+        std::cout << "T:" << (int)seconds \
+                  << "; A:" << data->dirents.size() \
+                  << "; RA:" << data->nbRedirectItems \
+                  << "; CA:" << data->nbCompItems \
+                  << "; UA:" << data->nbUnCompItems \
+                  << "; C:" << data->nbClusters \
+                  << "; CC:" << data->nbCompClusters \
+                  << "; UC:" << data->nbUnCompClusters \
+                  << "; WC:" << data->taskList.size() \
+                  << std::endl; \
+    }
+
+
 #define CLUSTER_BASE_OFFSET 1024
 
 namespace zim
 {
   namespace writer
   {
-    Creator::Creator(bool verbose, CompressionType c)
-      : verbose(verbose)
-      , compression(c)
+    Creator::Creator()
+      : m_clusterSize(DEFAULT_CLUSTER_SIZE)
     {}
-
     Creator::~Creator() = default;
 
-    void Creator::startZimCreation(const std::string& fname)
+    Creator& Creator::configVerbose(bool verbose)
     {
-      data = std::unique_ptr<CreatorData>(new CreatorData(fname, verbose, withIndex, indexingLanguage, compression));
-      data->setMinChunkSize(minChunkSize);
-
-      for(unsigned i=0; i<nbWorkerThreads; i++)
-      {
-        pthread_t thread;
-        pthread_create(&thread, NULL, taskRunner, this->data.get());
-        data->workerThreads.push_back(thread);
-      }
-
-      pthread_create(&data->writerThread, NULL, clusterWriter, this->data.get());
+      m_verbose = verbose;
+      return *this;
     }
 
-    void Creator::addArticle(std::shared_ptr<Article> article)
+    Creator& Creator::configCompression(Compression compression)
     {
-      auto dirent = data->createDirentFromArticle(article.get());
-      data->addDirent(dirent, article.get());
-      data->nbArticles++;
-      if (article->isRedirect()) {
-        data->nbRedirectArticles++;
-      } else {
-        if (article->shouldCompress())
-          data->nbCompArticles++;
-        else
-          data->nbUnCompArticles++;
-        if (!article->getFilename().empty())
-          data->nbFileArticles++;
-        if (article->shouldIndex())
-          data->nbIndexArticles++;
-      }
-      if (verbose && data->nbArticles%1000 == 0){
-        double seconds = difftime(time(NULL),data->start_time);
-        std::cout << "T:" << (int)seconds
-                  << "; A:" << data->nbArticles
-                  << "; RA:" << data->nbRedirectArticles
-                  << "; CA:" << data->nbCompArticles
-                  << "; UA:" << data->nbUnCompArticles
-                  << "; FA:" << data->nbFileArticles
-                  << "; IA:" << data->nbIndexArticles
-                  << "; C:" << data->nbClusters
-                  << "; CC:" << data->nbCompClusters
-                  << "; UC:" << data->nbUnCompClusters
-                  << "; WC:" << data->taskList.size()
+      if(compression == Compression::Lzma) {
+        std::cerr << "WARNING: LZMA compression method is deprecated."
+                  << " Support for it will be dropped from libzim soon."
                   << std::endl;
       }
+      m_compression = compression;
+      return *this;
+    }
 
-#if defined(ENABLE_XAPIAN)
-      if (article->shouldIndex()) {
-        data->titleIndexer.index(article.get());
-        if(withIndex && !article->isRedirect()) {
-          data->taskList.pushToQueue(new IndexTask(article));
-        }
+    Creator& Creator::configClusterSize(zim::size_type targetSize)
+    {
+      m_clusterSize = targetSize;
+      return *this;
+    }
+
+    Creator& Creator::configIndexing(bool indexing, const std::string& language)
+    {
+      m_withIndex = indexing;
+      m_indexingLanguage = language;
+      return *this;
+    }
+
+    Creator& Creator::configNbWorkers(unsigned nbWorkers)
+    {
+      m_nbWorkers = nbWorkers;
+      return *this;
+    }
+
+    void Creator::startZimCreation(const std::string& filepath)
+    {
+      data = std::unique_ptr<CreatorData>(
+        new CreatorData(filepath, m_verbose, m_withIndex, m_indexingLanguage, m_compression, m_clusterSize)
+      );
+
+      for(unsigned i=0; i<m_nbWorkers; i++)
+      {
+        std::thread thread(taskRunner, this->data.get());
+        data->workerThreads.push_back(std::move(thread));
       }
-#endif
+
+      data->writerThread = std::thread(clusterWriter, this->data.get());
+    }
+
+    void Creator::addItem(std::shared_ptr<Item> item)
+    {
+      bool compressContent = item->getAmendedHints()[COMPRESS];
+      auto dirent = data->createItemDirent(item.get());
+      data->addItemData(dirent, item->getContentProvider(), compressContent);
+      data->handle(dirent, item);
+
+      if (data->dirents.size()%1000 == 0) {
+        TPROGRESS();
+      }
+
+    }
+
+    void Creator::addMetadata(const std::string& name, const std::string& content, const std::string& mimetype)
+    {
+      auto provider = std::unique_ptr<ContentProvider>(new StringProvider(content));
+      addMetadata(name, std::move(provider), mimetype);
+    }
+
+    void Creator::addMetadata(const std::string& name, std::unique_ptr<ContentProvider> provider, const std::string& mimetype)
+    {
+      auto compressContent = isCompressibleMimetype(mimetype);
+      auto dirent = data->createDirent(NS::M, name, mimetype, "");
+      data->addItemData(dirent, std::move(provider), compressContent);
+      data->handle(dirent);
+    }
+
+    void Creator::addIllustration(unsigned int size, const std::string& content)
+    {
+      auto provider = std::unique_ptr<ContentProvider>(new StringProvider(content));
+      addIllustration(size, std::move(provider));
+    }
+
+    void Creator::addIllustration(unsigned int size, std::unique_ptr<ContentProvider> provider)
+    {
+      std::stringstream ss;
+      ss << "Illustration_" << size << "x" << size << "@1";
+      addMetadata(ss.str(), std::move(provider), "image/png");
+    }
+
+    void Creator::addRedirection(const std::string& path, const std::string& title, const std::string& targetPath, const Hints& hints)
+    {
+      auto dirent = data->createRedirectDirent(NS::C, path, title, NS::C, targetPath);
+      if (data->dirents.size()%1000 == 0){
+        TPROGRESS();
+      }
+
+      data->handle(dirent, hints);
     }
 
     void Creator::finishZimCreation()
     {
-      if (verbose) {
-        double seconds = difftime(time(NULL),data->start_time);
-        std::cout << "T:" << (int)seconds
-                  << "; A:" << data->nbArticles
-                  << "; RA:" << data->nbRedirectArticles
-                  << "; CA:" << data->nbCompArticles
-                  << "; UA:" << data->nbUnCompArticles
-                  << "; FA:" << data->nbFileArticles
-                  << "; IA:" << data->nbIndexArticles
-                  << "; C:" << data->nbClusters
-                  << "; CC:" << data->nbCompClusters
-                  << "; UC:" << data->nbUnCompClusters
-                  << "; WC:" << data->taskList.size()
-                  << std::endl;
+      // Create a redirection for the mainPage.
+      // We need to keep the created dirent to set the fileheader.
+      // Dirent doesn't have to be deleted.
+      if (!m_mainPath.empty()) {
+        data->mainPageDirent = data->createRedirectDirent(NS::W, "mainPage", "", NS::C, m_mainPath);
+        data->handle(data->mainPageDirent);
       }
 
-      // We need to wait that all indexation task has been done before closing the
-      // xapian database and add it to zim.
-      unsigned int wait = 0;
-      do {
-        microsleep(wait);
-        wait += 10;
-      } while(IndexTask::waiting_task.load() > 0);
+      TPROGRESS();
 
-#if defined(ENABLE_XAPIAN)
-      {
-        data->titleIndexer.indexingPostlude();
-        auto article = data->titleIndexer.getMetaArticle();
-        auto dirent = data->createDirentFromArticle(article);
-        data->addDirent(dirent, article);
-        delete article;
+      // mp_titleListingHandler is a special case, it have to handle all dirents (including itself)
+      for(auto& handler:data->m_direntHandlers) {
+        // This silently create all the needed dirents.
+        for(auto dirent:handler->getDirents()) {
+          data->mp_titleListingHandler->handle(dirent, Hints());
+        }
       }
-      if (withIndex) {
-        wait = 0;
-        do {
-          microsleep(wait);
-          wait += 10;
-        } while(IndexTask::waiting_task.load() > 0);
 
-        data->indexer->indexingPostlude();
-        microsleep(100);
-        auto article = data->indexer->getMetaArticle();
-        auto dirent = data->createDirentFromArticle(article);
-        data->addDirent(dirent, article);
-        delete article;
+      // Now we have all the dirents (but not the data), we must correctly set/fix the dirents
+      // before we ask data to the handlers
+      TINFO("ResolveRedirectIndexes");
+      data->resolveRedirectIndexes();
+
+      TINFO("Set entry indexes");
+      data->setEntryIndexes();
+
+      TINFO("Resolve mimetype");
+      data->resolveMimeTypes();
+
+      // We can now stop the direntHandlers, and get their content
+      bool titleListDirentSeen = false;
+      for(auto& handler:data->m_direntHandlers) {
+        handler->stop();
+        const auto& dirents = handler->getDirents();
+        if (dirents.empty()) {
+          continue;
+        }
+        auto providers = handler->getContentProviders();
+        ASSERT(dirents.size(), ==, providers.size());
+        auto provider_it = providers.begin();
+        for(auto& dirent:dirents) {
+          // As we use a "handler level" isCompressible, all content of the same handler
+          // must have the same compression.
+          data->addItemData(dirent, std::move(*provider_it), handler->isCompressible());
+          if (handler == data->mp_titleListingHandler && !titleListDirentSeen) {
+            // We have to get the offset of the titleList in the cluster before
+            // we close the cluster. Once the cluster is close, the offset information is dropped.
+            // This works only if titleListingHandler create the full (V0) titlelist in its first dirent.
+            data->m_titleListBlobOffset = data->uncompCluster->getBlobOffset(dirent->getBlobNumber());
+            titleListDirentSeen = true;
+          }
+          provider_it++;
+        }
       }
-#endif
 
-      // When we've seen all articles, write any remaining clusters.
+      // All the data has been added, we can now close all clusters
       if (data->compCluster->count())
         data->closeCluster(true);
 
@@ -198,87 +271,52 @@ namespace zim
 
       TINFO("Waiting for workers");
       // wait all cluster compression has been done
-      wait = 0;
+      unsigned int wait = 0;
       do {
         microsleep(wait);
         wait += 10;
       } while(ClusterTask::waiting_task.load() > 0);
 
-      // Quit all workerThreads
-      for (auto i=0U; i< nbWorkerThreads; i++) {
-        data->taskList.pushToQueue(nullptr);
-      }
-      for(auto& thread: data->workerThreads) {
-        pthread_join(thread, nullptr);
-      }
+      data->quitAllThreads();
 
-      // Wait for writerThread to finish.
-      data->clusterToWrite.pushToQueue(nullptr);
-      pthread_join(data->writerThread, nullptr);
+      // Delete all handler (they will clean there own data)
+      data->m_direntHandlers.clear();
 
-      TINFO("ResolveRedirectIndexes");
-      data->resolveRedirectIndexes();
-
-      TINFO("Set article indexes");
-      data->setArticleIndexes();
-
-      TINFO("Resolve mimetype");
-      data->resolveMimeTypes();
-
-      TINFO("create title index");
-      data->createTitleIndex();
       TINFO(data->dirents.size() << " title index created");
       TINFO(data->clustersList.size() << " clusters created");
 
       TINFO("write zimfile :");
-      write();
+      writeLastParts();
       ::close(data->out_fd);
 
       TINFO("rename tmpfile to final one.");
-      DEFAULTFS::rename(data->basename+".zim.tmp", data->basename+".zim");
+      DEFAULTFS::rename(data->tmpFileName, data->zimName);
 
       TINFO("finish");
     }
 
     void Creator::fillHeader(Fileheader* header) const
     {
-      auto mainUrl = getMainUrl();
-      auto layoutUrl = getLayoutUrl();
+      header->setMainPage(
+        data->mainPageDirent
+        ? entry_index_type(data->mainPageDirent->getIdx())
+        : std::numeric_limits<entry_index_type>::max());
+      header->setLayoutPage(std::numeric_limits<entry_index_type>::max());
 
-      if (data->isExtended) {
-        header->setMajorVersion(Fileheader::zimExtendedMajorVersion);
-      } else {
-        header->setMajorVersion(Fileheader::zimClassicMajorVersion);
-      }
-      header->setMinorVersion(Fileheader::zimMinorVersion);
-      header->setMainPage(std::numeric_limits<article_index_type>::max());
-      header->setLayoutPage(std::numeric_limits<article_index_type>::max());
-
-      if (!mainUrl.empty() || !layoutUrl.empty())
-      {
-        for (auto& dirent: data->dirents)
-        {
-          if (mainUrl == dirent->getFullUrl())
-          {
-            header->setMainPage(article_index_type(dirent->getIdx()));
-          }
-
-          if (layoutUrl == dirent->getFullUrl())
-          {
-            header->setLayoutPage(article_index_type(dirent->getIdx()));
-          }
-        }
-      }
-
-      header->setUuid( getUuid() );
+      header->setUuid( m_uuid );
       header->setArticleCount( data->dirents.size() );
 
       header->setMimeListPos( Fileheader::size );
 
+      // We assume here that titleListingHandler create the V0 listing in its first dirent.
+      auto cluster = data->mp_titleListingHandler->getDirents()[0]->getCluster();
+      header->setTitleIdxPos(
+        offset_type(cluster->getOffset() + cluster->getDataOffset() + data->m_titleListBlobOffset));
+
       header->setClusterCount( data->clustersList.size() );
     }
 
-    void Creator::write() const
+    void Creator::writeLastParts() const
     {
       Fileheader header;
       fillHeader(&header);
@@ -313,15 +351,6 @@ namespace zim
         _write(out_fd, tmp_buff, sizeof(offset_type));
       }
 
-      TINFO(" write title index");
-      header.setTitleIdxPos(lseek(out_fd, 0, SEEK_CUR));
-      for (Dirent* dirent: data->titleIdx)
-      {
-        char tmp_buff[sizeof(article_index_type)];
-        toLittleEndian(dirent->getIdx().v, tmp_buff);
-        _write(out_fd, tmp_buff, sizeof(article_index_type));
-      }
-
       TINFO(" write cluster offset list");
       header.setClusterPtrPos(lseek(out_fd, 0, SEEK_CUR));
       for (auto cluster : data->clustersList)
@@ -362,39 +391,36 @@ namespace zim
                                    bool verbose,
                                    bool withIndex,
                                    std::string language,
-                                   CompressionType c)
-      : compression(c),
+                                   Compression c,
+                                   size_t clusterSize)
+      : mainPageDirent(nullptr),
+        compression(c),
+        zimName(fname),
+        tmpFileName(fname + ".tmp"),
+        clusterSize(clusterSize),
         withIndex(withIndex),
         indexingLanguage(language),
-#if defined(ENABLE_XAPIAN)
-        titleIndexer(language, IndexingMode::TITLE, true),
-#endif
         verbose(verbose),
-        nbArticles(0),
-        nbRedirectArticles(0),
-        nbCompArticles(0),
-        nbUnCompArticles(0),
-        nbFileArticles(0),
-        nbIndexArticles(0),
+        nbRedirectItems(0),
+        nbCompItems(0),
+        nbUnCompItems(0),
         nbClusters(0),
         nbCompClusters(0),
         nbUnCompClusters(0),
         start_time(time(NULL))
     {
-      basename =  (fname.size() > 4 && fname.compare(fname.size() - 4, 4, ".zim") == 0)
-                        ? fname.substr(0, fname.size() - 4)
-                        : fname;
-      auto zim_name = basename + ".zim.tmp";
 #ifdef _WIN32
-int mode =  _S_IREAD | _S_IWRITE;
+      int flag = _O_RDWR | _O_CREAT | _O_TRUNC | _O_BINARY;
+      int mode =  _S_IREAD | _S_IWRITE;
 #else
+      int flag = O_RDWR | O_CREAT | O_TRUNC;
       mode_t mode = S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH;
 #endif
-      out_fd = open(zim_name.c_str(), O_RDWR|O_CREAT|O_TRUNC, mode);
+      out_fd = open(tmpFileName.c_str(), flag, mode);
       if (out_fd == -1){
         perror(nullptr);
         std::ostringstream ss;
-        ss << "Cannot create file " << zim_name;
+        ss << "Cannot create file " << tmpFileName;
         throw std::runtime_error(ss.str());
       }
       if(lseek(out_fd, CLUSTER_BASE_OFFSET, SEEK_SET) != CLUSTER_BASE_OFFSET) {
@@ -408,15 +434,20 @@ int mode =  _S_IREAD | _S_IWRITE;
       // to track the dirents currently in each, so we can fix up the
       // cluster index if the other one ends up written first.
       compCluster = new Cluster(compression);
-      uncompCluster = new Cluster(zimcompNone);
+      uncompCluster = new Cluster(Compression::None);
 
 #if defined(ENABLE_XAPIAN)
-      titleIndexer.indexingPrelude(basename+"_title.idx");
-      if (withIndex) {
-          indexer = new XapianIndexer(indexingLanguage, IndexingMode::FULL, true);
-          indexer->indexingPrelude(basename+".idx");
-      }
+      auto xapianIndexer = std::make_shared<XapianHandler>(this, withIndex);
+      m_direntHandlers.push_back(xapianIndexer);
 #endif
+
+      mp_titleListingHandler = std::make_shared<TitleListingHandler>(this);
+      m_direntHandlers.push_back(mp_titleListingHandler);
+      m_direntHandlers.push_back(std::make_shared<CounterHandler>(this));
+
+      for(auto& handler:m_direntHandlers) {
+        handler->start();
+      }
     }
 
     CreatorData::~CreatorData()
@@ -428,13 +459,27 @@ int mode =  _S_IREAD | _S_IWRITE;
       for(auto& cluster: clustersList) {
         delete cluster;
       }
-#if defined(ENABLE_XAPIAN)
-      if (indexer)
-        delete indexer;
-#endif
+      quitAllThreads();
     }
 
-    void CreatorData::addDirent(Dirent* dirent, const Article* article)
+    void CreatorData::quitAllThreads() {
+      // Quit all workerThreads
+      for (auto i=0U; i< workerThreads.size(); i++) {
+        taskList.pushToQueue(nullptr);
+      }
+      for(auto& thread: workerThreads) {
+        thread.join();
+      }
+      workerThreads.clear();
+
+      // Wait for writerThread to finish.
+      if (writerThread.joinable()) {
+        clusterToWrite.pushToQueue(nullptr);
+        writerThread.join();
+      }
+    }
+
+    void CreatorData::addDirent(Dirent* dirent)
     {
       auto ret = dirents.insert(dirent);
       if (!ret.second) {
@@ -444,7 +489,7 @@ int mode =  _S_IREAD | _S_IWRITE;
           dirents.erase(ret.first);
           dirents.insert(dirent);
         } else {
-          std::cerr << "Impossible to add " << dirent->getFullUrl().getLongUrl() << std::endl;
+          std::cerr << "Impossible to add " << NsAsChar(dirent->getNamespace()) << "/" << dirent->getPath() << std::endl;
           std::cerr << "  dirent's title to add is : " << dirent->getTitle() << std::endl;
           std::cerr << "  existing dirent's title is : " << existing->getTitle() << std::endl;
           return;
@@ -455,70 +500,67 @@ int mode =  _S_IREAD | _S_IWRITE;
       if (dirent->isRedirect())
       {
         unresolvedRedirectDirents.insert(dirent);
+        nbRedirectItems++;
         return;
       }
+    }
 
+    void CreatorData::addItemData(Dirent* dirent, std::unique_ptr<ContentProvider> provider, bool compressContent)
+    {
       // Add blob data to compressed or uncompressed cluster.
-      auto articleSize = article->getSize();
-      if (articleSize > 0)
+      auto itemSize = provider->getSize();
+      if (itemSize > 0)
       {
         isEmpty = false;
       }
 
-      Cluster *cluster;
-      if (article->shouldCompress())
-      {
-        cluster = compCluster;
-      }
-      else
-      {
-        cluster = uncompCluster;
-      }
+      auto cluster = compressContent ? compCluster : uncompCluster;
 
       // If cluster will be too large, write it to dis, and open a new
       // one for the content.
       if ( cluster->count()
-        && cluster->size().v+articleSize >= minChunkSize * 1024
+        && cluster->size().v+itemSize >= clusterSize
          )
       {
-        log_info("cluster with " << cluster->count() << " articles, " <<
+        log_info("cluster with " << cluster->count() << " items, " <<
                  cluster->size() << " bytes; current title \"" <<
                  dirent->getTitle() << '\"');
-        cluster = closeCluster(article->shouldCompress());
+        cluster = closeCluster(compressContent);
       }
 
       dirent->setCluster(cluster);
-      cluster->addArticle(article);
+      cluster->addContent(std::move(provider));
+
+      if (compressContent) {
+        nbCompItems++;
+      } else {
+        nbUnCompItems++;
+      }
+
     }
 
-    Dirent* CreatorData::createDirentFromArticle(const Article* article)
+    Dirent* CreatorData::createDirent(NS ns, const std::string& path, const std::string& mimetype, const std::string& title)
     {
-      auto dirent = pool.getDirent();
-      dirent->setUrl(article->getUrl());
-      dirent->setTitle(article->getTitle());
+      auto dirent = pool.getClassicDirent(ns, path, title, getMimeTypeIdx(mimetype));
+      addDirent(dirent);
+      return dirent;
+    }
 
-      if (article->isRedirect())
-      {
-        dirent->setRedirect(nullptr);
-        dirent->setRedirectUrl(article->getRedirectUrl());
-      }
-      else if (article->isLinktarget())
-      {
-        dirent->setLinktarget();
-      }
-      else if (article->isDeleted())
-      {
-        dirent->setDeleted();
-      }
-      else
-      {
-        auto mimetype = article->getMimeType();
-        if (mimetype.empty()) {
-          std::cerr << "Warning, " << article->getUrl().getLongUrl() << " have empty mimetype." << std::endl;
-          mimetype = "application/octet-stream";
-        }
-        dirent->setMimeType(getMimeTypeIdx(mimetype));
+    Dirent* CreatorData::createItemDirent(const Item* item)
+    {
+      auto path = item->getPath();
+      auto mimetype = item->getMimeType();
+      if (mimetype.empty()) {
+        std::cerr << "Warning, " << item->getPath() << " have empty mimetype." << std::endl;
+        mimetype = "application/octet-stream";
       }
+      return createDirent(NS::C, item->getPath(), mimetype, item->getTitle());
+    }
+
+    Dirent* CreatorData::createRedirectDirent(NS ns, const std::string& path, const std::string& title, NS targetNs, const std::string& targetPath)
+    {
+      auto dirent = pool.getRedirectDirent(ns, path, title, targetNs, targetPath);
+      addDirent(dirent);
       return dirent;
     }
 
@@ -539,22 +581,20 @@ int mode =  _S_IREAD | _S_IWRITE;
       taskList.pushToQueue(new ClusterTask(cluster));
       clusterToWrite.pushToQueue(cluster);
 
-      if (cluster->is_extended() )
-        isExtended = true;
       if (compressed)
       {
         cluster = compCluster = new Cluster(compression);
       } else {
-        cluster = uncompCluster = new Cluster(zimcompNone);
+        cluster = uncompCluster = new Cluster(Compression::None);
       }
       return cluster;
     }
 
-    void CreatorData::setArticleIndexes()
+    void CreatorData::setEntryIndexes()
     {
       // set index
       INFO("set index");
-      article_index_t idx(0);
+      entry_index_t idx(0);
       for (auto& dirent: dirents) {
         dirent->setIdx(idx);
         idx += 1;
@@ -567,24 +607,24 @@ int mode =  _S_IREAD | _S_IWRITE;
       INFO("Resolve redirect");
       for (auto dirent: unresolvedRedirectDirents)
       {
-        Dirent tmpDirent(dirent->getRedirectUrl());
+        Dirent tmpDirent(dirent->getRedirectNs(), dirent->getRedirectPath());
         auto target_pos = dirents.find(&tmpDirent);
         if(target_pos == dirents.end()) {
-          INFO("Invalid redirection " << dirent->getFullUrl().getLongUrl() << " redirecting to (missing) " << dirent->getRedirectUrl().getLongUrl());
+          INFO("Invalid redirection "
+              << NsAsChar(dirent->getNamespace()) << '/' << dirent->getPath()
+              << " redirecting to (missing) "
+              << NsAsChar(dirent->getRedirectNs()) << '/' << dirent->getRedirectPath());
           dirents.erase(dirent);
+          dirent->markRemoved();
+          if (dirent == mainPageDirent) {
+            mainPageDirent = nullptr;
+          }
         } else  {
           dirent->setRedirect(*target_pos);
         }
       }
     }
 
-    void CreatorData::createTitleIndex()
-    {
-      titleIdx.clear();
-      for (auto dirent: dirents)
-        titleIdx.insert(dirent);
-    }
-
     void CreatorData::resolveMimeTypes()
     {
       std::vector<std::string> oldMImeList;
@@ -610,7 +650,7 @@ int mode =  _S_IREAD | _S_IWRITE;
 
       for (auto& dirent: dirents)
       {
-        if (dirent->isArticle())
+        if (dirent->isItem())
           dirent->setMimeType(mapping[dirent->getMimeType()]);
       }
     }
