@@ -29,12 +29,12 @@
 // #include "utf8convert.h"
 
 #include <algorithm>
+#include <mutex>
 
 #include <ctype.h>
 #include <cstring>
 #include <stdio.h>
 #include <stdlib.h>
-#include <pthread.h>
 
 using namespace std;
 
@@ -47,7 +47,7 @@ lowercase_string(string &str)
 }
 
 map<string, unsigned int> zim::HtmlParser::named_ents;
-static pthread_mutex_t sInitLock = PTHREAD_MUTEX_INITIALIZER;
+static std::mutex sInitLock;
 
 inline static bool
 p_notdigit(char c)
@@ -107,7 +107,7 @@ zim::HtmlParser::HtmlParser()
 #include "namedentities.h"
 	{ NULL, 0 }
     };
-    pthread_mutex_lock(&sInitLock);
+    std::lock_guard<std::mutex> l(sInitLock);
     if (named_ents.empty()) {
 	const struct ent *i = ents;
 	while (i->n) {
@@ -115,7 +115,6 @@ zim::HtmlParser::HtmlParser()
 	    ++i;
 	}
     }
-    pthread_mutex_unlock(&sInitLock);
 }
 
 void
