@@ -45,6 +45,7 @@
 #include "gtest/gtest.h"
 
 #include <zim/zim.h>
+#include <zim/writer/contentProvider.h>
 
 #include "../src/buffer.h"
 #include "../src/cluster.h"
@@ -65,7 +66,7 @@ using zim::unittests::write_to_buffer;
 
 TEST(ClusterTest, create_cluster)
 {
-  zim::writer::Cluster cluster(zim::zimcompNone);
+  zim::writer::Cluster cluster(zim::Compression::None);
 
   ASSERT_EQ(cluster.count().v, 0U);
 
@@ -73,9 +74,9 @@ TEST(ClusterTest, create_cluster)
   std::string blob1("ABCDEFGHIJKLMNOPQRSTUVWXYZ");
   std::string blob2("abcdefghijklmnopqrstuvwxyz");
 
-  cluster.addData(blob0.data(), zim::zsize_t(blob0.size()));
-  cluster.addData(blob1.data(), zim::zsize_t(blob1.size()));
-  cluster.addData(blob2.data(), zim::zsize_t(blob2.size()));
+  cluster.addContent(blob0);
+  cluster.addContent(blob1);
+  cluster.addContent(blob2);
 
   ASSERT_EQ(cluster.count().v, 3U);
   ASSERT_EQ(cluster.getBlobSize(zim::blob_index_t(0)).v, blob0.size());
@@ -85,21 +86,21 @@ TEST(ClusterTest, create_cluster)
 
 TEST(ClusterTest, read_write_cluster)
 {
-  zim::writer::Cluster cluster(zim::zimcompNone);
+  zim::writer::Cluster cluster(zim::Compression::None);
 
   std::string blob0("123456789012345678901234567890");
   std::string blob1("ABCDEFGHIJKLMNOPQRSTUVWXYZ");
   std::string blob2("abcdefghijklmnop vwxyz");
 
-  cluster.addData(blob0.data(), zim::zsize_t(blob0.size()));
-  cluster.addData(blob1.data(), zim::zsize_t(blob1.size()));
-  cluster.addData(blob2.data(), zim::zsize_t(blob2.size()));
+  cluster.addContent(blob0);
+  cluster.addContent(blob1);
+  cluster.addContent(blob2);
 
   cluster.close();
   auto buffer = write_to_buffer(cluster);
   const auto cluster2shptr = zim::Cluster::read(zim::BufferReader(buffer), zim::offset_t(0));
   zim::Cluster& cluster2 = *cluster2shptr;
-  ASSERT_EQ(cluster2.getCompression(), zim::zimcompNone);
+  ASSERT_EQ(cluster2.getCompression(), zim::Compression::None);
   ASSERT_EQ(cluster2.isExtended, false);
   ASSERT_EQ(cluster2.count().v, 3U);
   ASSERT_EQ(cluster2.getBlobSize(zim::blob_index_t(0)).v, blob0.size());
@@ -107,19 +108,34 @@ TEST(ClusterTest, read_write_cluster)
   ASSERT_EQ(cluster2.getBlobSize(zim::blob_index_t(2)).v, blob2.size());
 }
 
+TEST(ClusterTest, read_write_no_content)
+{
+  zim::writer::Cluster cluster(zim::Compression::None);
+
+  cluster.close();
+  auto buffer = write_to_buffer(cluster, "\3garbage");
+  const auto cluster2shptr = zim::Cluster::read(zim::BufferReader(buffer), zim::offset_t(0));
+  zim::Cluster& cluster2 = *cluster2shptr;
+  ASSERT_EQ(cluster2.getCompression(), zim::Compression::None);
+  ASSERT_EQ(cluster2.isExtended, false);
+  ASSERT_EQ(cluster2.count().v, 0U);
+}
+
 TEST(ClusterTest, read_write_empty)
 {
-  zim::writer::Cluster cluster(zim::zimcompNone);
+  zim::writer::Cluster cluster(zim::Compression::None);
 
-  cluster.addData(0, zim::zsize_t(0));
-  cluster.addData(0, zim::zsize_t(0));
-  cluster.addData(0, zim::zsize_t(0));
+  std::string emptyString;
+
+  cluster.addContent(emptyString);
+  cluster.addContent(emptyString);
+  cluster.addContent(emptyString);
 
   cluster.close();
   auto buffer = write_to_buffer(cluster);
   const auto cluster2shptr = zim::Cluster::read(zim::BufferReader(buffer), zim::offset_t(0));
   zim::Cluster& cluster2 = *cluster2shptr;
-  ASSERT_EQ(cluster2.getCompression(), zim::zimcompNone);
+  ASSERT_EQ(cluster2.getCompression(), zim::Compression::None);
   ASSERT_EQ(cluster2.isExtended, false);
   ASSERT_EQ(cluster2.count().v, 3U);
   ASSERT_EQ(cluster2.getBlobSize(zim::blob_index_t(0)).v, 0U);
@@ -129,15 +145,15 @@ TEST(ClusterTest, read_write_empty)
 
 TEST(ClusterTest, read_write_clusterLzma)
 {
-  zim::writer::Cluster cluster(zim::zimcompLzma);
+  zim::writer::Cluster cluster(zim::Compression::Lzma);
 
   std::string blob0("123456789012345678901234567890");
   std::string blob1("ABCDEFGHIJKLMNOPQRSTUVWXYZ");
   std::string blob2("abcdefghijklmnopqrstuvwxyz");
 
-  cluster.addData(blob0.data(), zim::zsize_t(blob0.size()));
-  cluster.addData(blob1.data(), zim::zsize_t(blob1.size()));
-  cluster.addData(blob2.data(), zim::zsize_t(blob2.size()));
+  cluster.addContent(blob0);
+  cluster.addContent(blob1);
+  cluster.addContent(blob2);
 
   cluster.close();
   auto buffer = write_to_buffer(cluster);
@@ -145,7 +161,7 @@ TEST(ClusterTest, read_write_clusterLzma)
   zim::Cluster& cluster2 = *cluster2shptr;
   ASSERT_EQ(cluster2.isExtended, false);
   ASSERT_EQ(cluster2.count().v, 3U);
-  ASSERT_EQ(cluster2.getCompression(), zim::zimcompLzma);
+  ASSERT_EQ(cluster2.getCompression(), zim::Compression::Lzma);
   ASSERT_EQ(cluster2.getBlobSize(zim::blob_index_t(0)).v, blob0.size());
   ASSERT_EQ(cluster2.getBlobSize(zim::blob_index_t(1)).v, blob1.size());
   ASSERT_EQ(cluster2.getBlobSize(zim::blob_index_t(2)).v, blob2.size());
@@ -156,15 +172,15 @@ TEST(ClusterTest, read_write_clusterLzma)
 
 TEST(ClusterTest, read_write_clusterZstd)
 {
-  zim::writer::Cluster cluster(zim::zimcompZstd);
+  zim::writer::Cluster cluster(zim::Compression::Zstd);
 
   std::string blob0("123456789012345678901234567890");
   std::string blob1("ABCDEFGHIJKLMNOPQRSTUVWXYZ");
   std::string blob2("abcdefghijklmnopqrstuvwxyz");
 
-  cluster.addData(blob0.data(), zim::zsize_t(blob0.size()));
-  cluster.addData(blob1.data(), zim::zsize_t(blob1.size()));
-  cluster.addData(blob2.data(), zim::zsize_t(blob2.size()));
+  cluster.addContent(blob0);
+  cluster.addContent(blob1);
+  cluster.addContent(blob2);
 
   cluster.close();
   auto buffer = write_to_buffer(cluster);
@@ -172,7 +188,7 @@ TEST(ClusterTest, read_write_clusterZstd)
   zim::Cluster& cluster2 = *cluster2shptr;
   ASSERT_EQ(cluster2.isExtended, false);
   ASSERT_EQ(cluster2.count().v, 3U);
-  ASSERT_EQ(cluster2.getCompression(), zim::zimcompZstd);
+  ASSERT_EQ(cluster2.getCompression(), zim::Compression::Zstd);
   ASSERT_EQ(cluster2.getBlobSize(zim::blob_index_t(0)).v, blob0.size());
   ASSERT_EQ(cluster2.getBlobSize(zim::blob_index_t(1)).v, blob1.size());
   ASSERT_EQ(cluster2.getBlobSize(zim::blob_index_t(2)).v, blob2.size());
@@ -181,7 +197,31 @@ TEST(ClusterTest, read_write_clusterZstd)
   ASSERT_EQ(blob2, std::string(cluster2.getBlob(zim::blob_index_t(2))));
 }
 
-#if !defined(__APPLE__)
+class FakeProvider : public zim::writer::ContentProvider
+{
+  public:
+    FakeProvider(zim::size_type size)
+      : size(size),
+        offset(0),
+        buffer(new char[1024*1024U])
+    {
+      memset(buffer.get(), 0, 1024*1024U);
+    }
+
+    zim::size_type getSize() const { return size; }
+    zim::Blob feed() {
+      auto outSize = std::min(zim::size_type(1024*1024), size-offset);
+      auto blob = zim::Blob(buffer.get(), outSize);
+      offset += outSize;
+      return blob;
+    }
+
+  private:
+    zim::size_type size;
+    zim::offset_type offset;
+    std::unique_ptr<char[]> buffer;
+};
+
 TEST(ClusterTest, read_write_extended_cluster)
 {
   //zim::writer doesn't suport 32 bits architectures.
@@ -191,6 +231,7 @@ TEST(ClusterTest, read_write_extended_cluster)
 
   char* SKIP_BIG_MEMORY_TEST = std::getenv("SKIP_BIG_MEMORY_TEST");
   if (SKIP_BIG_MEMORY_TEST != nullptr && std::string(SKIP_BIG_MEMORY_TEST) == "1") {
+    std::cout << "Skip big memory test" << std::endl;
     return;
   }
 
@@ -198,60 +239,50 @@ TEST(ClusterTest, read_write_extended_cluster)
   std::string blob0("123456789012345678901234567890");
   std::string blob1("ABCDEFGHIJKLMNOPQRSTUVWXYZ");
   std::string blob2("abcdefghijklmnopqrstuvwxyz");
-  zim::size_type bigger_than_4g = 1024LL*1024LL*1024LL*4LL+1024LL;
+  const uint64_t FOUR_GIB = 4LL * 1024LL*1024LL*1024LL;
+  zim::size_type almost_4g = FOUR_GIB - 16;
+  auto bigProvider = std::unique_ptr<zim::writer::ContentProvider>(new FakeProvider(almost_4g));
+  std::string blob4("zyxwvutsrqponmlkjihgfedcba");
 
-  auto buffer = zim::Buffer::makeBuffer(nullptr, zim::zsize_t(0));
-  {
-    char* blob3 = nullptr;
-    try {
-      blob3 = new char[bigger_than_4g];
-      // MEM = 4GiB
-    } catch (std::bad_alloc& e) {
-      // Not enough memory, we cannot test cluster bigger than 4Go :(
-      return;
-    }
+  zim::writer::Cluster cluster(zim::Compression::None);
+  cluster.addContent(blob0);
+  cluster.addContent(blob1);
+  cluster.addContent(blob2);
+  cluster.addContent(std::move(bigProvider));
+  cluster.addContent(blob4);
 
-    {
-      zim::writer::Cluster cluster(zim::zimcompNone);
-      cluster.addData(blob0.data(), zim::zsize_t(blob0.size()));
-      cluster.addData(blob1.data(), zim::zsize_t(blob1.size()));
-      cluster.addData(blob2.data(), zim::zsize_t(blob2.size()));
-      try {
-        cluster.addData(blob3, zim::zsize_t(bigger_than_4g));
-        // MEM = 8GiB
-      } catch (std::bad_alloc& e) {
-        // Not enough memory, we cannot test cluster bigger than 4Go :(
-        delete[] blob3;
-        return;
-      }
-      ASSERT_EQ(cluster.is_extended(), true);
+  ASSERT_GT(cluster.size().v, FOUR_GIB);
+  ASSERT_EQ(cluster.is_extended(), true);
 
-      delete[] blob3;
-      // MEM = 4GiB
+  auto buffer = write_to_buffer(cluster);
+  // 4GiB
 
-      cluster.close();
-      buffer = write_to_buffer(cluster);
-    }
-  }
   const auto cluster2shptr = zim::Cluster::read(zim::BufferReader(buffer), zim::offset_t(0));
   zim::Cluster& cluster2 = *cluster2shptr;
   ASSERT_EQ(cluster2.isExtended, true);
-  ASSERT_EQ(cluster2.count().v, 4U);
-  ASSERT_EQ(cluster2.getCompression(), zim::zimcompNone);
+  ASSERT_EQ(cluster2.count().v, 5U);
+  ASSERT_EQ(cluster2.getCompression(), zim::Compression::None);
   ASSERT_EQ(cluster2.getBlobSize(zim::blob_index_t(0)).v, blob0.size());
   ASSERT_EQ(cluster2.getBlobSize(zim::blob_index_t(1)).v, blob1.size());
   ASSERT_EQ(cluster2.getBlobSize(zim::blob_index_t(2)).v, blob2.size());
-  ASSERT_EQ(cluster2.getBlobSize(zim::blob_index_t(3)).v, bigger_than_4g);
+  ASSERT_EQ(cluster2.getBlobSize(zim::blob_index_t(3)).v, almost_4g);
   ASSERT_EQ(blob0, std::string(cluster2.getBlob(zim::blob_index_t(0))));
   ASSERT_EQ(blob1, std::string(cluster2.getBlob(zim::blob_index_t(1))));
   ASSERT_EQ(blob2, std::string(cluster2.getBlob(zim::blob_index_t(2))));
+  ASSERT_EQ(blob4, std::string(cluster2.getBlob(zim::blob_index_t(4))));
 }
-#endif
+
 
 TEST(ClusterTest, read_extended_cluster)
 {
-  std::FILE* tmpfile = std::tmpfile();
-  int fd = fileno(tmpfile);
+  char* SKIP_BIG_MEMORY_TEST = std::getenv("SKIP_BIG_MEMORY_TEST");
+  if (SKIP_BIG_MEMORY_TEST != nullptr && std::string(SKIP_BIG_MEMORY_TEST) == "1") {
+    std::cout << "Skip big memory test" << std::endl;
+    return;
+  }
+
+  TempFile tmpfile("extended_cluster");
+  int fd = tmpfile.fd();
   ssize_t bytes_written;
 
   std::string blob0("123456789012345678901234567890");
@@ -305,15 +336,14 @@ TEST(ClusterTest, read_extended_cluster)
 //  std::fseek(tmpfile, bigger_than_4g-1, SEEK_CUR);
   a = '\0';
   bytes_written = write(fd, &a, 1);
-  fflush(tmpfile);
+  tmpfile.close();
 
-  auto filePart = new zim::FilePart<>(fileno(tmpfile));
-  auto fileCompound = std::shared_ptr<zim::FileCompound>(new zim::FileCompound(filePart));
-  const auto cluster2shptr = zim::Cluster::read(zim::FileReader(fileCompound), zim::offset_t(0));
+  auto fileCompound = std::make_shared<zim::FileCompound>(tmpfile.path());
+  const auto cluster2shptr = zim::Cluster::read(zim::MultiPartFileReader(fileCompound), zim::offset_t(0));
   zim::Cluster& cluster2 = *cluster2shptr;
   ASSERT_EQ(cluster2.isExtended, true);
   ASSERT_EQ(cluster2.count().v, 4U);
-  ASSERT_EQ(cluster2.getCompression(), zim::zimcompNone);
+  ASSERT_EQ(cluster2.getCompression(), zim::Compression::None);
   ASSERT_EQ(cluster2.getBlobSize(zim::blob_index_t(0)).v, blob0.size());
   ASSERT_EQ(cluster2.getBlobSize(zim::blob_index_t(1)).v, blob1.size());
   ASSERT_EQ(cluster2.getBlobSize(zim::blob_index_t(2)).v, blob2.size());
@@ -331,8 +361,6 @@ TEST(ClusterTest, read_extended_cluster)
   } else {
     ASSERT_EQ(b.size(), bigger_than_4g);
   }
-
-  fclose(tmpfile);
 }
 
 
